/************************************************************************************************
 *	Name:			Avalanche_Beacon.h															*
 *	Author:			J. Mikulastik																*
 *	Created:		2014																		*
 *	Platform:		AVR Atmega32																*
 *  Description:	Avalanche beacon, receiver,transmitter										*
 *	Xtal:			Internal 1MHz																*
 ***********************************************************************************************/ 



#ifndef AVALANCHE_BEACON_H_
#define AVALANCHE_BEACON_H_

#define F_CPU	8000000UL
#define XTAL	F_CPU


#include <avr/io.h>
#include <util/delay.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <avr/interrupt.h>
//#include <string.h>
#include <avr/pgmspace.h>
#include "pcd8544.h"
#include "DELAY.H"


/*******************************************TYPEDEF**********************************************/
typedef enum
{
	ARR_LEFT = 1,
	ARR_RIGHT = 2,
	ARR_CENTER = 3
}LcdArrow;

/*******************************************MACROS**********************************************/
//State of SM
#define ST_INIT		0
#define ST_TRANS	1
#define ST_REC		2
#define	ST_ERR		3

//IO pin def
#define RSIG		PA1			//Ain: Received signal
#define BAT			PA3			//Ain: Battery voltage
#define TSIG		PA4			//Ain: Antena receiving power
#define BUTT2		PA6			//Din: button decoder 2
#define BUTT1		PA7			//Din: button decoder 1

#define BUTTI		PB2			//Din: External interrupt from button decoder
#define ATT			PB3			//Dout: Received signal attenuator
#define BL			PB4			//Dout: LCD BackLight

#define MOD			PC2			//Dout: Signal modulation
#define ANTBOT		PC3			//Dout: bottom antena select
#define THERM		PC4			//Dout: Dallas 1-wire
#define LED2		PC5			//Dout: Led2
#define LED1R		PC6			//Dout: Led1 red
#define LED1G		PC7			//Dout: Led1 green

#define GPS_EN		PD2			//Dout: GPS module enable
#define VDRV		PD3			//Dout: Suplly rec. IO
#define ANTTOP		PD4			//Dout: top antena select
#define SIRENA		PD5			//Dout: TIM1 pwm
#define SW			PD7			//Din: switch Receive or Transmitt

// 
#define BLINKX		3				//uP led blink every x second
#define	INTFREG		500				// const interrupt freg
#define INTDC		50				//trensmition 0-500
#define BUTTHOLDTIME 5				//button holdon time agains the oscilation 
#define	ADCBUFFSIZE	6				//

#define	ADCANTENA	4				//ADC chanels
#define ADCBAT		3
#define ADCRSSI		1

//Setter/Getter macros
#define GETBUTT1		PINA & (1<<BUTT1)
#define GETBUTT2		PINA & (1<<BUTT2)
#define GETBUTTI		PINB & (1<<BUTTI)
#define GETSW			PIND & (1<<SW)

#define SETMOD			PORTC|=(1<<MOD)
#define CLEARMOD		PORTC&=~(1<<MOD)
#define TOGGLEMOD		PORTC^= (1 << MOD)

#define CLEARLED1G		PORTC|=(1<<LED1R)
#define SETLED1G		PORTC&=~(1<<LED1R)
#define TOGGLELED1G		PORTC^= (1 << LED1R)
#define CLEARLED1R		PORTC|=(1<<LED1G)
#define SETLED1R		PORTC&=~(1<<LED1G)
#define TOGGLELED1R		PORTC^= (1 << LED1G)

#define SETLED2			PORTC|=(1<<LED2)
#define CLEARLED2		PORTC&=~(1<<LED2)
#define TOGGLELED2		PORTC^= (1 << LED2)
 			
#define SETATT			PORTB|=(1<<ATT)
#define CLEARATT		PORTB&=~(1<<ATT)

#define SETBL			PORTB|=(1<<BL)
#define CLEARBL			PORTB&=~(1<<BL)

#define SETDISBOT		PORTC|=(1<<ANTBOT)
#define CLEARDISBOT		PORTC&=~(1<<ANTBOT)

#define SETDISTOP		PORTD|=(1<<ANTTOP)
#define CLEARDISTOP		PORTD&=~(1<<ANTTOP)

#define SETGPSEN		PORTD|=(1<<GPS_EN)
#define CLEARGPSEN		PORTD&=~(1<<GPS_EN)

#define SETVDRV			PORTD|=(1<<VDRV)
#define CLEARVDRV		PORTD&=~(1<<VDRV)

#define SETSIRENA		 TCCR1A |=(1<<COM1A1)
#define CLEARSIRENA		 TCCR1A &=~(1<<COM1A1)


//THERMOMETER

#define	BAUD	19200
#define bauddivider (uint)(1.0 * XTAL / BAUD / 16 - 0.5)

#define uchar unsigned char
#define uint unsigned int
#define bit uchar
#define idata
#define code

#define W1_PIN	PC4
#define W1_IN	PINC
#define W1_OUT	PORTC
#define W1_DDR	DDRC

#include "1WIRE.H"
#include "TEMPMEAS.H"




/*************************************FUNCTION*DEFINITION**************************************/

//State machine function
unsigned char FC_Init(void);
unsigned char FC_Trans(void);
unsigned char FC_Rec(void);
unsigned char FC_Err(void);
//Transient function
unsigned char FC_Init2Trans(void);
unsigned char FC_Init2Rec(void);
unsigned char FC_Trans2Rec(void);
unsigned char FC_Rec2Trans(void);
//Peripherial function
void Set_IO_Initial_Cond (void);
void Set_Arrow (LcdArrow arrow);
void Set_IO_Reset (void);
void PWM_Init (void);
void PWM_setDutyA (double x);
void Tim0_Init (void);
void Ext_Int_Init (void);
void Draw_Transmitt(LcdPixelMode p);
void ADC_Init (void);
void Adc_Init_Rec(void);
uint16_t adc_read(uint8_t ch);




/********************************************IMAGES********************************************/
static const unsigned char waitImage[] PROGMEM = {
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xC0,0x20,0x20,
	0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0xC0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0xF0,0x10,0x10,0x10,0x10,0xE0,0x00,0x00,0xF0,0x00,0x00,0x80,0x40,0x40,
	0x40,0x80,0x00,0x00,0x80,0x40,0x40,0x40,0x80,0x00,0x00,0x80,0x40,0x40,0x40,0x80,
	0x00,0x00,0x80,0x40,0x40,0x40,0x80,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x0F,0x11,0x31,0x31,0xD1,0xF1,0xD1,0xD1,0x31,0x11,0x11,0x0F,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x1F,0x01,0x01,0x01,0x01,0x00,0x00,0x00,0x1F,0x00,
	0x00,0x0F,0x12,0x12,0x12,0x0B,0x00,0x00,0x0C,0x12,0x12,0x0A,0x1F,0x00,0x00,0x09,
	0x12,0x12,0x12,0x0C,0x00,0x00,0x0F,0x12,0x12,0x12,0x0B,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0xF0,0x18,0x86,0x86,0xE1,0xF1,0xE1,0xE1,0x86,0x18,0x18,
	0xF0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x60,0x80,0x00,0x80,
	0x60,0x80,0x00,0x80,0x60,0x00,0x40,0x20,0x20,0x20,0xC0,0x00,0x00,0xE8,0x00,0x20,
	0xF8,0x20,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x03,0x07,0x07,0x07,0x07,0x07,0x07,
	0x07,0x07,0x07,0x07,0x03,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x03,0x0C,0x03,0x00,0x03,0x0C,0x03,0x00,0x00,0x06,0x09,0x09,0x05,0x0F,0x00,
	0x00,0x0F,0x00,0x00,0x0F,0x08,0x00,0x00,0x00,0x00,0x08,0x00,0x00,0x08,0x00,0x00,
	0x08,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};


static const unsigned char photo[] PROGMEM = {
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x80,0x80,0x80,0x80,
	0xC0,0xC0,0x80,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0xE0,0xF0,0xF8,0xFC,0xFE,0x7E,0x7F,0x7F,
	0x7F,0x7F,0x7F,0xFF,0xFF,0xFF,0xFF,0xFF,0xFC,0xF0,0xE0,0xC0,0x80,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xD8,0xFE,0xFF,0xFF,0x7F,0x43,0x21,
	0x30,0xB0,0x20,0x00,0x00,0x00,0x00,0x00,0x20,0x20,0xB1,0x33,0x27,0xFF,0xFF,0x7F,
	0x1F,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x05,0xFF,0x1F,
	0x00,0x00,0x00,0x00,0x01,0x03,0x01,0x00,0xC0,0x20,0x80,0xC0,0x00,0x01,0x03,0x01,
	0x00,0x81,0x7F,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x03,0x7C,0xA0,0x10,0x80,0x00,0xB0,0x30,0xE0,0xE1,0xE1,0xF1,0x71,0x31,
	0xB0,0x18,0x90,0x48,0xE0,0x1F,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x40,0x20,
	0x20,0x20,0x10,0x10,0x10,0x10,0x08,0x08,0x0B,0x1E,0x3C,0x7B,0xE0,0xD6,0xF0,0xF9,
	0xFD,0xFC,0xB0,0xC9,0x74,0x79,0x3E,0x39,0x08,0x10,0x10,0x10,0x20,0x40,0x80,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};





#endif /* AVALANCHE_BEACON_H_ */